import numpy as np
import matplotlib.pyplot as plt

plt.rcParams.update({"text.usetex": True, "font.family": "sans-serif", "font.sans-serif": ["Helvetica"]})


def get_vals(vec_x, vec_coeff, vec_t):
    assert vec_x.ndim == 1 and vec_coeff.ndim == 1
    N = vec_x.size
    assert N >= 1 and vec_coeff.size == N
    vals = np.zeros(vec_t.shape)
    for k in range(N - 1):
        vals = (t - vec_x[-(k + 2)]) * (vec_coeff[-(k + 1)] + vals)
    vals += vec_coeff[0]
    return vals


def get_divdiff_tab(x, y):
    assert x.ndim == 1 and y.ndim == 1 and x.shape == y.shape
    N = x.size
    assert N >= 2
    tab = np.zeros((N + 1, N))
    tab[0, :] = x
    tab[1, :] = y
    for k in range(1, N):
        delta_y = tab[k, 1:(N - k + 1)] - tab[k, 0:(N - k)]  # length N-k
        delta_x = x[k:] - x[:N - k]  # Length N-k
        new_row = delta_y / delta_x  # Pointwise division
        tab[1 + k, :N - k] = new_row
    return tab


N = 6
vec_x1 = np.linspace(-1., 1., N)
vec_y1 = 1.0 / (1.0 + 25. * vec_x1 * vec_x1)
tab = get_divdiff_tab(vec_x1, vec_y1)
vec_coeff1 = tab[1:, 0]

N = 8
vec_x2 = np.linspace(-1., 1., N)
vec_y2 = 1.0 / (1.0 + 25. * vec_x2 * vec_x2)
tab = get_divdiff_tab(vec_x2, vec_y2)
vec_coeff2 = tab[1:, 0]

N = 10
vec_x3 = np.linspace(-1., 1., N)
vec_y3 = 1.0 / (1.0 + 25. * vec_x3 * vec_x3)
tab = get_divdiff_tab(vec_x3, vec_y3)
vec_coeff3 = tab[1:, 0]

N = 12
vec_x4 = np.linspace(-1., 1., N)
vec_y4 = 1.0 / (1.0 + 25. * vec_x4 * vec_x4)
tab = get_divdiff_tab(vec_x4, vec_y4)
vec_coeff4 = tab[1:, 0]

pp = 512
fig, ax = plt.subplots()
t = np.linspace(-1., 1., pp)
data = 1.0 / (1.0 + 25. * t * t)
ax.plot(t, data, label='Truth')
data = get_vals(vec_x1, vec_coeff1, t)
ax.plot(t, data, label='$P_5$')
data = get_vals(vec_x2, vec_coeff2, t)
ax.plot(t, data, label='$P_7$')
data = get_vals(vec_x3, vec_coeff3, t)
ax.plot(t, data, label='$P_9$')
data = get_vals(vec_x4, vec_coeff4, t)
ax.plot(t, data, label='$P_{11}$')
ax.grid()
ax.legend()
plt.savefig("Runge.eps", format='eps')